﻿Module Module1
    Sub Main()
        ' Natürliche Zahlen - Die nächste Zahl ist die aktuelle plus eins
        Dim Naturals = 1.Generate(Function(n) n + 1)

        ' Quadratzahlen - Hier wird Linq verwendet
        Dim Squares = From n In Naturals Select n ^ 2

        ' Fibonacci-Zahlen - Jede Zahl ist die Summe ihrer beiden Vorgänger.
        '  Hierzu müssen die Vorgänger (a und b) zwischengespeichert werden, in 
        '  diesem Falle geschieht dies in einem anonymen Typen. Das 
        '  letztendliche Ergebnis ist wiederum nur eine Zahl, die aus den 
        '  Paaren extrahiert wird
        Dim Fibonacci = (New With {.a = 1, .b = 1}).Generate( _
            Successor:=Function(p) New With {.a = p.b, .b = p.a + p.b}, _
            Selector:=Function(p) p.a)

        ' Dreieckszahlen - Jede Zahl ist die Summe aller Zahlen bis zu dieser
        '  Das Vorgehen ist hier ähnlich zu dem bei den Fibonacci-Zahlen, hier 
        '  jedoch in verkürzter Syntax
        Dim TriangleNumbers = (New With {.Sum = 1, .i = 1}).Generate( _
            Function(p) New With {.Sum = p.Sum + p.i, .i = p.i + 1}, _
                Function(p) p.Sum)

        ' Ausgeben
        Console.WriteLine("Natürliche Zahlen:   {0}", Naturals.Show(8))
        Console.WriteLine("Quadratzahlen:       {0}", Squares.Show(8))
        Console.WriteLine("Fibonaccizahlen:     {0}", Fibonacci.Show(8))
        Console.WriteLine("Dreieckszahlen:      {0}", TriangleNumbers.Show(8))

        Console.ReadKey()
    End Sub

End Module
